@@ -3,7 +3,7 @@
 #ifndef SCM_TAGS_H
 #define SCM_TAGS_H
 
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002,2003,2004,2008,2009
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002,2003,2004,2008,2009,2010
  * Free Software Foundation, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -411,18 +411,18 @@ typedef scm_t_uintptr scm_t_bits;
 #define scm_tc7_stringbuf       39
 #define scm_tc7_bytevector	77
 
-#define scm_tc7_unused_1	31
+#define scm_tc7_foreign		31
 #define scm_tc7_hashtable	29
 #define scm_tc7_fluid		37
 #define scm_tc7_dynamic_state	45
 
-#define scm_tc7_unused_4	47
-#define scm_tc7_unused_5	53
-#define scm_tc7_unused_6	55
-#define scm_tc7_unused_7	71
+#define scm_tc7_frame		47
+#define scm_tc7_objcode		53
+#define scm_tc7_vm		55
+#define scm_tc7_vm_cont		71
 
 #define scm_tc7_unused_17	61
-#define scm_tc7_gsubr		63
+#define scm_tc7_unused_21	63
 #define scm_tc7_unused_19	69
 #define scm_tc7_program		79
 #define scm_tc7_unused_9	85
@@ -648,11 +648,6 @@ enum scm_tc8_tags
   case scm_tc3_struct + 112:\
   case scm_tc3_struct + 120
 
-/* For subrs
- */
-#define scm_tcs_subrs \
-  case scm_tc7_gsubr
-
 
 
 #if (SCM_ENABLE_DEPRECATED == 1)
