@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2009 Free Software Foundation, Inc.
+/* Copyright (C) 2001, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -34,17 +34,16 @@
 
 
 static SCM
-VM_NAME (struct scm_vm *vp, SCM program, SCM *argv, int nargs)
+VM_NAME (SCM vm, SCM program, SCM *argv, int nargs)
 {
   /* VM registers */
   register scm_t_uint8 *ip IP_REG;	/* instruction pointer */
   register SCM *sp SP_REG;		/* stack pointer */
   register SCM *fp FP_REG;		/* frame pointer */
+  struct scm_vm *vp = SCM_VM_DATA (vm);
 
   /* Cache variables */
   struct scm_objcode *bp = NULL;	/* program base pointer */
-  SCM *free_vars = NULL;                /* free variables */
-  size_t free_vars_count = 0;           /* length of FREE_VARS */
   SCM *objects = NULL;			/* constant objects */
   size_t object_count = 0;              /* length of OBJECTS */
   SCM *stack_limit = vp->stack_limit;	/* stack limit address */
@@ -53,10 +52,6 @@ VM_NAME (struct scm_vm *vp, SCM program, SCM *argv, int nargs)
   int nvalues = 0;
   SCM finish_args;                      /* used both for returns: both in error
                                            and normal situations */
-#if VM_USE_HOOKS
-  SCM hook_args = SCM_EOL;
-#endif
-
 #ifdef HAVE_LABELS_AS_VALUES
   static void **jump_table = NULL;
 #endif
@@ -133,6 +128,8 @@ VM_NAME (struct scm_vm *vp, SCM program, SCM *argv, int nargs)
   {
     SCM err_msg;
 
+    /* FIXME: need to sync regs before allocating anything, in each case. */
+
   vm_error_bad_instruction:
     err_msg  = scm_from_locale_string ("VM: Bad instruction: ~s");
     finish_args = scm_list_1 (scm_from_uchar (ip[-1]));
@@ -148,19 +145,24 @@ VM_NAME (struct scm_vm *vp, SCM program, SCM *argv, int nargs)
     goto vm_error;
 
   vm_error_kwargs_length_not_even:
-    err_msg  = scm_from_locale_string ("Bad keyword argument list: odd length");
-    finish_args = SCM_EOL;
-    goto vm_error;
+    SYNC_ALL ();
+    err_msg = scm_from_locale_string ("Odd length of keyword argument list");
+    scm_error_scm (sym_keyword_argument_error, program, err_msg,
+                   SCM_EOL, SCM_BOOL_F);
 
   vm_error_kwargs_invalid_keyword:
-    err_msg  = scm_from_locale_string ("Bad keyword argument list: expected keyword");
-    finish_args = SCM_EOL;
-    goto vm_error;
+    /* FIXME say which one it was */
+    SYNC_ALL ();
+    err_msg = scm_from_locale_string ("Invalid keyword");
+    scm_error_scm (sym_keyword_argument_error, program, err_msg,
+                   SCM_EOL, SCM_BOOL_F);
 
   vm_error_kwargs_unrecognized_keyword:
-    err_msg  = scm_from_locale_string ("Bad keyword argument list: unrecognized keyword");
-    finish_args = SCM_EOL;
-    goto vm_error;
+    /* FIXME say which one it was */
+    SYNC_ALL ();
+    err_msg = scm_from_locale_string ("Unrecognized keyword");
+    scm_error_scm (sym_keyword_argument_error, program, err_msg,
+                   SCM_EOL, SCM_BOOL_F);
 
   vm_error_too_many_args:
     err_msg  = scm_from_locale_string ("VM: Too many arguments");
