@@ -1,4 +1,4 @@
-/* Copyright (C) 2001, 2009 Free Software Foundation, Inc.
+/* Copyright (C) 2001, 2009, 2010 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -28,7 +28,6 @@
 #include <gc/gc_mark.h>
 
 #include "_scm.h"
-#include "vm-bootstrap.h"
 #include "frames.h"
 #include "instructions.h"
 #include "objcodes.h"
@@ -72,7 +71,13 @@
  * VM Continuation
  */
 
-scm_t_bits scm_tc16_vm_cont;
+void
+scm_i_vm_cont_print (SCM x, SCM port, scm_print_state *pstate)
+{
+  scm_puts ("#<vm-continuation ", port);
+  scm_uintprint (SCM_UNPACK (x), 16, port);
+  scm_puts (">", port);
+}
 
 static SCM
 capture_vm_cont (struct scm_vm *vp)
@@ -92,7 +97,7 @@ capture_vm_cont (struct scm_vm *vp)
   p->fp = vp->fp;
   memcpy (p->stack_base, vp->stack_base, p->stack_size * sizeof (SCM));
   p->reloc = p->stack_base - vp->stack_base;
-  SCM_RETURN_NEWSMOB (scm_tc16_vm_cont, p);
+  return scm_cell (scm_tc7_vm_cont, (scm_t_bits)p);
 }
 
 static void
@@ -144,26 +149,24 @@ scm_vm_reinstate_continuations (SCM conts)
     reinstate_vm_cont (SCM_VM_DATA (SCM_CAAR (conts)), SCM_CDAR (conts));
 }
 
-static void enfalsen_frame (void *p)
-{ 
-  struct scm_vm *vp = p;
-  vp->trace_frame = SCM_BOOL_F;
-}
-
 static void
-vm_dispatch_hook (struct scm_vm *vp, SCM hook, SCM hook_args)
+vm_dispatch_hook (SCM vm, int hook_num)
 {
-  if (!scm_is_false (vp->trace_frame))
+  struct scm_vm *vp;
+  SCM hook;
+  SCM frame;
+
+  vp = SCM_VM_DATA (vm);
+  hook = vp->hooks[hook_num];
+
+  if (SCM_LIKELY (scm_is_false (hook))
+      || scm_is_null (SCM_HOOK_PROCEDURES (hook)))
     return;
-
-  scm_dynwind_begin (0);
-  /* FIXME, stack holder should be the vm */
-  vp->trace_frame = scm_c_make_frame (SCM_BOOL_F, vp->fp, vp->sp, vp->ip, 0);
-  scm_dynwind_unwind_handler (enfalsen_frame, vp, SCM_F_WIND_EXPLICITLY);
-
-  scm_c_run_hook (hook, hook_args);
-
-  scm_dynwind_end ();
+  
+  vp->trace_level--;
+  frame = scm_c_make_frame (vm, vp->fp, vp->sp, vp->ip, 0);
+  scm_c_run_hookn (hook, &frame, 1);
+  vp->trace_level++;
 }
 
 
@@ -171,9 +174,18 @@ vm_dispatch_hook (struct scm_vm *vp, SCM hook, SCM hook_args)
  * VM Internal functions
  */
 
-static SCM sym_vm_run;
-static SCM sym_vm_error;
-static SCM sym_debug;
+SCM_SYMBOL (sym_vm_run, "vm-run");
+SCM_SYMBOL (sym_vm_error, "vm-error");
+SCM_SYMBOL (sym_keyword_argument_error, "keyword-argument-error");
+SCM_SYMBOL (sym_debug, "debug");
+
+void
+scm_i_vm_print (SCM x, SCM port, scm_print_state *pstate)
+{
+  scm_puts ("#<vm ", port);
+  scm_uintprint (SCM_UNPACK (x), 16, port);
+  scm_puts (">", port);
+}
 
 static SCM
 really_make_boot_program (long nargs)
@@ -193,8 +205,8 @@ really_make_boot_program (long nargs)
   bp->len = sizeof(text);
   bp->metalen = 0;
 
-  u8vec = scm_take_u8vector ((scm_t_uint8*)bp,
-                             sizeof (struct scm_objcode) + sizeof (text));
+  u8vec = scm_c_take_bytevector ((scm_t_int8*)bp,
+                                 sizeof (struct scm_objcode) + sizeof (text));
   ret = scm_make_program (scm_bytecode_to_objcode (u8vec),
                           SCM_BOOL_F, SCM_BOOL_F);
   SCM_SET_CELL_WORD_0 (ret, SCM_CELL_WORD_0 (ret) | SCM_F_PROGRAM_IS_BOOT);
@@ -261,41 +273,6 @@ resolve_variable (SCM what, SCM program_module)
     }
 }
   
-static SCM
-apply_foreign (SCM proc, SCM *args, int nargs, int headroom)
-{
-  SCM_ASRTGO (SCM_NIMP (proc), badproc);
-
-  switch (SCM_TYP7 (proc))
-    {
-    case scm_tc7_smob:
-      if (!SCM_SMOB_APPLICABLE_P (proc))
-        goto badproc;
-      switch (nargs)
-        {
-        case 0:
-          return SCM_SMOB_APPLY_0 (proc);
-        case 1:
-          return SCM_SMOB_APPLY_1 (proc, args[0]);
-        case 2:
-          return SCM_SMOB_APPLY_2 (proc, args[0], args[1]);
-        default:
-          {
-            SCM arglist = SCM_EOL;
-            while (nargs-- > 2)
-              arglist = scm_cons (args[nargs], arglist);
-            return SCM_SMOB_APPLY_3 (proc, args[0], args[1], arglist);
-          }
-        }
-    case scm_tc7_gsubr:
-      return scm_i_gsubr_apply_array (proc, args, nargs, headroom);
-    default:
-    badproc:
-      scm_wrong_type_arg ("apply", SCM_ARG1, proc);
-    }
-}
-
-
 #define VM_DEFAULT_STACK_SIZE	(64 * 1024)
 
 #define VM_NAME   vm_regular_engine
@@ -317,8 +294,6 @@ apply_foreign (SCM proc, SCM *args, int nargs, int headroom)
 static const scm_t_vm_engine vm_engines[] = 
   { vm_regular_engine, vm_debug_engine };
 
-scm_t_bits scm_tc16_vm;
-
 #ifdef VM_ENABLE_PRECISE_STACK_GC_SCAN
 
 /* The GC "kind" for the VM stack.  */
@@ -333,16 +308,13 @@ make_vm (void)
   int i;
   struct scm_vm *vp;
 
-  if (!scm_tc16_vm)
-    return SCM_BOOL_F; /* not booted yet */
-
   vp = scm_gc_malloc (sizeof (struct scm_vm), "vm");
 
   vp->stack_size  = VM_DEFAULT_STACK_SIZE;
 
 #ifdef VM_ENABLE_PRECISE_STACK_GC_SCAN
-  vp->stack_base = GC_generic_malloc (vp->stack_size * sizeof (SCM),
-				      vm_stack_gc_kind);
+  vp->stack_base = (SCM *)
+    GC_generic_malloc (vp->stack_size * sizeof (SCM), vm_stack_gc_kind);
 
   /* Keep a pointer to VP so that `vm_stack_mark ()' can know what the stack
      top is.  */
@@ -363,10 +335,10 @@ make_vm (void)
   vp->fp    	  = NULL;
   vp->engine      = SCM_VM_DEBUG_ENGINE;
   vp->options     = SCM_EOL;
+  vp->trace_level = 0;
   for (i = 0; i < SCM_VM_NUM_HOOKS; i++)
     vp->hooks[i] = SCM_BOOL_F;
-  vp->trace_frame = SCM_BOOL_F;
-  SCM_RETURN_NEWSMOB (scm_tc16_vm, vp);
+  return scm_cell (scm_tc7_vm, (scm_t_bits)vp);
 }
 #undef FUNC_NAME
 
@@ -406,12 +378,13 @@ SCM
 scm_c_vm_run (SCM vm, SCM program, SCM *argv, int nargs)
 {
   struct scm_vm *vp = SCM_VM_DATA (vm);
-  return vm_engines[vp->engine](vp, program, argv, nargs);
+  return vm_engines[vp->engine](vm, program, argv, nargs);
 }
 
-SCM
-scm_vm_apply (SCM vm, SCM program, SCM args)
-#define FUNC_NAME "scm_vm_apply"
+SCM_DEFINE (scm_vm_apply, "vm-apply", 3, 0, 0,
+            (SCM vm, SCM program, SCM args),
+            "")
+#define FUNC_NAME s_scm_vm_apply
 {
   SCM *argv;
   int i, nargs;
@@ -618,13 +591,24 @@ SCM_DEFINE (scm_set_vm_option_x, "set-vm-option!", 3, 0, 0,
 }
 #undef FUNC_NAME
 
-SCM_DEFINE (scm_vm_trace_frame, "vm-trace-frame", 1, 0, 0,
+SCM_DEFINE (scm_vm_trace_level, "vm-trace-level", 1, 0, 0,
 	    (SCM vm),
 	    "")
-#define FUNC_NAME s_scm_vm_trace_frame
+#define FUNC_NAME s_scm_vm_trace_level
 {
   SCM_VALIDATE_VM (1, vm);
-  return SCM_VM_DATA (vm)->trace_frame;
+  return scm_from_int (SCM_VM_DATA (vm)->trace_level);
+}
+#undef FUNC_NAME
+
+SCM_DEFINE (scm_set_vm_trace_level_x, "set-vm-trace-level!", 2, 0, 0,
+	    (SCM vm, SCM level),
+	    "")
+#define FUNC_NAME s_scm_set_vm_trace_level_x
+{
+  SCM_VALIDATE_VM (1, vm);
+  SCM_VM_DATA (vm)->trace_level = scm_to_int (level);
+  return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
@@ -644,34 +628,9 @@ SCM scm_load_compiled_with_vm (SCM file)
 void
 scm_bootstrap_vm (void)
 {
-  static int strappage = 0;
-  
-  if (strappage)
-    return;
-
-  scm_bootstrap_frames ();
-  scm_bootstrap_instructions ();
-  scm_bootstrap_objcodes ();
-  scm_bootstrap_programs ();
-
-  scm_tc16_vm_cont = scm_make_smob_type ("vm-cont", 0);
-
-  scm_tc16_vm = scm_make_smob_type ("vm", 0);
-  scm_set_smob_apply (scm_tc16_vm, scm_vm_apply, 1, 0, 1);
-
-  scm_c_define ("load-compiled",
-                scm_c_make_gsubr ("load-compiled/vm", 1, 0, 0,
-                                  scm_load_compiled_with_vm));
-
-  sym_vm_run = scm_from_locale_symbol ("vm-run");
-  sym_vm_error = scm_from_locale_symbol ("vm-error");
-  sym_debug = scm_from_locale_symbol ("debug");
-
   scm_c_register_extension ("libguile", "scm_init_vm",
                             (scm_t_extension_init_func)scm_init_vm, NULL);
 
-  strappage = 1;
-
 #ifdef VM_ENABLE_PRECISE_STACK_GC_SCAN
   vm_stack_gc_kind =
     GC_new_kind (GC_new_free_list (),
@@ -684,8 +643,6 @@ scm_bootstrap_vm (void)
 void
 scm_init_vm (void)
 {
-  scm_bootstrap_vm ();
-
 #ifndef SCM_MAGIC_SNARFER
 #include "libguile/vm.x"
 #endif
