@@ -1,6 +1,6 @@
 ;;; Repl commands
 
-;; Copyright (C) 2001, 2009 Free Software Foundation, Inc.
+;; Copyright (C) 2001, 2009, 2010 Free Software Foundation, Inc.
 
 ;; This library is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU Lesser General Public
@@ -29,13 +29,14 @@
   #:use-module (system vm vm)
   #:autoload (system base language) (lookup-language language-reader)
   #:autoload (system vm debug) (vm-debugger vm-backtrace)
-  #:autoload (system vm trace) (vm-trace vm-trace-on vm-trace-off)
+  #:autoload (system vm trace) (vm-trace)
   #:autoload (system vm profile) (vm-profile)
   #:use-module (ice-9 format)
   #:use-module (ice-9 session)
   #:use-module (ice-9 documentation)
   #:use-module (ice-9 and-let-star)
   #:use-module (ice-9 rdelim)
+  #:use-module (statprof)
   #:export (meta-command))
 
 
@@ -227,13 +228,7 @@ List/show/set options."
      (display (repl-option-ref repl key))
      (newline))
     ((,key ,val)
-     (repl-option-set! repl key val)
-     (case key
-       ((trace)
-        (let ((vm (repl-vm repl)))
-          (if val
-              (apply vm-trace-on vm val)
-              (vm-trace-off vm))))))))
+     (repl-option-set! repl key val))))
 
 (define-meta-command (quit repl)
   "quit
@@ -359,13 +354,14 @@ Time execution."
 	    (get identity gc-start gc-end))
     result))
 
-(define-meta-command (profile repl form . opts)
+(define-meta-command (profile repl (form) . opts)
   "profile FORM
 Profile execution."
-  (apply vm-profile
-         (repl-vm repl)
-         (repl-compile repl (repl-parse repl form))
-         opts))
+  ;; FIXME opts
+  (let ((vm (repl-vm repl))
+        (proc (make-program (repl-compile repl (repl-parse repl form)))))
+    (apply statprof (lambda () (vm-apply vm proc '())) opts)))
+
 
 
 ;;;
@@ -384,13 +380,11 @@ Start debugger."
 
 (define-meta-command (trace repl form . opts)
   "trace FORM
-Trace execution.
-
-  -s    Display stack
-  -l    Display local variables
-  -b    Bytecode level trace"
-  (apply vm-trace (repl-vm repl)
-         (repl-compile repl (repl-parse repl form))
+Trace execution."
+  ;; FIXME: doc options, or somehow deal with them better
+  (apply vm-trace
+         (repl-vm repl)
+         (make-program (repl-compile repl (repl-parse repl form)))
          opts))
 
 (define-meta-command (step repl)
